﻿// analyse.js
// date: /^(\d{4}|\d{2})\/(\d{2})\/(\d{2})/
// time: /(\d{2}):(\d{2}):(\d{2}).(\d{2})$/
//       /(\d{2}):(\d{2}):(\d{2})$/
//       /(\d{2}):(\d{2})$/

/**
 * @fileOverview [スレッド情報...]で表示される画面を扱います。
 */
 
/**
 * スレッドから情報を解析してまとめて表示します。
 * @static
 */
var Analyse = {
	/** [スレッド情報] ダイアログを表示します。 */
	analyse: function() {
		dialogAnalyse.clear();
		// -------------
		dialogAnalyse.addPage ("Thread", "スレッド", SKIN_PATH + "img/thread.png");
		dialogAnalyse.addFrame("Thread", "Summary", "概要");
		dialogAnalyse.addLabel("Thread", "Summary", "", "タイトル:");
		dialogAnalyse.addLabel("Thread", "Summary", "", ThreadDocument.title,null,1);
		dialogAnalyse.addLabel("Thread", "Summary", "", "URL:");
		dialogAnalyse.addLink ("Thread", "Summary", "", EXACT_URL, EXACT_URL,null,1);
		dialogAnalyse.addLabel("Thread", "Summary", "", "総レス数:");
		dialogAnalyse.addLabel("Thread", "Summary", "", ThreadDocument.countAll,null,1);
		dialogAnalyse.addLabel("Thread", "Summary", "", "期間:");
		var lastIndex = (ThreadDocument.countAll > 1000) ? 1000 : ThreadDocument.countAll;
		dialogAnalyse.addLabel("Thread", "Summary", "", this.getDate(1).toLocaleString() + " ～ " + this.getDate(lastIndex).toLocaleString(),null,1);
		dialogAnalyse.addLabel("Thread", "Summary", "", "(" + this.toStringfromTimeDiff((this.getDate(lastIndex).getTime() - this.getDate(1).getTime())) + ")",null,1);
		dialogAnalyse.addLabel("Thread", "Summary", "", "勢い:");
		dialogAnalyse.addLabel("Thread", "Summary", "", this.getPaceString(),null,1);
		// -------------
		dialogAnalyse.addPage ("User", "中の人", SKIN_PATH + "img/user.png");
		dialogAnalyse.addFrame("User", "Name", "名前",96);
		var nameArray = [];
		var nameTable = [];
		var arrayIndex = 0;
		var nameItems = ResNodes.getNames();
		for (var i = 0; i < nameItems.length; i++) {
			var node = nameItems.items(i);
			var name = node.textContent;
			if (nameTable["_" + name]) {
				nameArray[nameTable["_" + name]].count++;
			} else {
				nameTable["_" + name] = arrayIndex;
				nameArray[arrayIndex] = {name: name, count: 1};
				arrayIndex++;
			}
		}
		nameArray.sort(function(a,b){return b.count - a.count});
		for (var i = 0; i < nameArray.length; i++) {
			if (nameArray[i].count > 1) dialogAnalyse.addLabel("User", "Name", "", nameArray[i].name + "(" + nameArray[i].count + ")");
		}
		// ---
		dialogAnalyse.addFrame("User", "ID", "ID",96);
		var idArray = [];
		for (var id in ID.items) {
			idArray.push({id: id, count: ID.items[id].length});
		}
		idArray.sort(function(a,b){return b.count - a.count});
		dialogAnalyse.addLabel("User", "ID", "", "合計 " + idArray.length + " 個のID");
		for (var i = 0; i < idArray.length; i++) {
			if (idArray[i].count > 1) dialogAnalyse.addID("User", "ID", "", idArray[i].id, idArray[i].count);
		}
		// ---
		dialogAnalyse.addFrame("User", "Trackback", "人気レス",96);
		var trackbackArray = [];
		Trackback.traverse();
		for (var index in Trackback.items) {
			trackbackArray.push({index: index, count: Trackback.items[index].length});
		}
		trackbackArray.sort(function(a,b){return b.count - a.count});
		var hasbeenAdded = false;
		for (var i = 0; i < trackbackArray.length; i++) {
			if (trackbackArray[i].count > 1) {
				dialogAnalyse.addTrackback("User", "Trackback", "", trackbackArray[i].index, trackbackArray[i].count);
				hasbeenAdded = true;
			}
		}
		if (!hasbeenAdded) dialogAnalyse.addLabel("User", "Trackback", "", "なし");
		// -------------
		dialogAnalyse.addPage ("Link", "リンク", SKIN_PATH + "img/external.png");
		dialogAnalyse.addFrame("Link", "2ch", "2ch", 128);
		dialogAnalyse.addFrame("Link", "External", "外部サイト",128);
		dialogAnalyse.addPage ("Image", "画像", SKIN_PATH + "img/image.png");
		dialogAnalyse.addFrame("Image", "Image", "一覧",256);
		dialogAnalyse.addPage ("Video", "動画", SKIN_PATH + "img/video.png");
		dialogAnalyse.addFrame("Video", "Video", "一覧",256);
		var externalLinks = ResNodes.getOutLinks();
		var urlTable = [];
		var imageCount = 0;
		var videoCount = 0;
		var extCount = 0;
		var chCount = 0;
		for (var i = 0; i < externalLinks.length; i++) {
			var node = externalLinks.items(i);
			var href = node.rel;
			if (urlTable[href]) { continue } else { urlTable[href] = true };
			var container = ResNodes.getParentContainer(node);
			var index = ResNodes.getIndexByContainer(container);
			if (ImagePopup.isImage(href)) {
				imageCount++;
				dialogAnalyse.addLink ("Image", "Image", "", href, node.href, index);
			} else if (VideoPopup.getVideoSource(href)) {
				videoCount++;
				dialogAnalyse.addLink ("Video", "Video", "", href, node.href, index);
			} else {
				if (!href.match(/\.2ch\.net\/|\.bbspink\.com\/|\.machi\.to\/|jbbs\.livedoor\.jp\//)) {
					extCount++;
					dialogAnalyse.addLink ("Link", "External", "", href, node.href, index);
				} else {
					chCount++;
					dialogAnalyse.addLink ("Link", "2ch", "", href, node.href, index);
				}
			}
		}
		if (!imageCount) dialogAnalyse.addLabel ("Image", "Image", "", "なし");
		if (!videoCount) dialogAnalyse.addLabel ("Video", "Video", "", "なし");
		if (!extCount)   dialogAnalyse.addLabel ("Link", "External", "", "なし");
		if (!chCount)    dialogAnalyse.addLabel ("Link", "2ch", "", "なし");
		// -------------
		dialogAnalyse.show();
	},
	/** 指定されたレス番号の日付を、日付型に変換して返します。
	 * @param  {Number} index レス番号
	 * @return {Date} 日付
	 */
	getDate: function(index) {
		var str = Nodes.getDate(index).textContent;
		str.match(/^(\d{4}|\d{2})\/(\d{2})\/(\d{2})/);
		var fullYear  = parseInt(RegExp.$1);
		if (fullYear < 100) fullYear += (fullYear >= 90) ? 1900 : 2000;
		var date = new Date(RegExp.$2 + "/" + RegExp.$3 + "/" + fullYear);
		if (str.match(/(\d{2}):(\d{2}):(\d{2}).(\d{2})(\s|$)/)) {
			date.setHours(parseInt(RegExp.$1));
			date.setMinutes(parseInt(RegExp.$2));
			date.setSeconds(parseInt(RegExp.$3));
			date.setMilliseconds(parseInt(RegExp.$4));
		} else if (str.match(/(\d{2}):(\d{2}):(\d{2})(\s|$)/)) {
			date.setHours(parseInt(RegExp.$1));
			date.setMinutes(parseInt(RegExp.$2));
			date.setSeconds(parseInt(RegExp.$3));
		} else if (str.match(/(\d{2}):(\d{2})(\s|$)/)) {
			date.setHours(parseInt(RegExp.$1));
			date.setMinutes(parseInt(RegExp.$2));
		}
		return date;
	},
	/** ミリ秒を日本語の時間表記に変換します。
	 * @param  {Number} arg 時間（ミリ秒）
	 * @return {String} 変換された文字列
	 */
	toStringfromTimeDiff: function(arg){
		var strReturn = new String();
		var valDate = Math.floor(arg / 31536000000);
		if (valDate > 0) strReturn += valDate + "年";
		var valDate = Math.floor((arg % 31536000000) / 2628000000);
		if (valDate > 0) strReturn += valDate + "ヶ月";
		var valDate = Math.floor(((arg % 31536000000) % 2628000000) / 86400000);
		if (valDate > 0) strReturn += valDate + "日 ";
		var valDate = Math.floor((((arg % 31536000000) % 2628000000) % 86400000) / 3600000) ;
		if (valDate > 0) strReturn += valDate + "時間";
		var valDate = Math.floor(((((arg % 31536000000) % 2628000000) % 86400000) % 3600000) / 60000);
		if (valDate > 0) strReturn += valDate + "分";
		if (strReturn.length == 0) {
			var valDate = Math.floor((((((arg % 31536000000) % 2628000000) % 86400000) % 3600000) % 60000) / 1000 * 1000) / 1000;
		} else {
			var valDate = Math.floor((((((arg % 31536000000) % 2628000000) % 86400000) % 3600000) % 60000) / 1000);
		}
		if (valDate > 0) strReturn += valDate + "秒";
		return strReturn;
	},
	/** スレッドの勢いを取得します。
	 * @return {String} 勢い
	 */
	getPaceString: function(){
		var lastIndex = (ThreadDocument.countAll > 1000) ? 1000 : ThreadDocument.countAll;
		return this.toStringfromTimeDiff((this.getDate(lastIndex).getTime() - this.getDate(1).getTime()) / lastIndex) + " / レス";
	}
};

/**
 * [スレッド情報] ダイアログを管理します。
 * @static
 */
var dialogAnalyse = {
	/** ダイアログの親要素
	 * @type element
	 */
	content: document.createElement("div"),
	/** [OK] ボタンが押されたときの処理をします。*/
	onOK: function(){
		this.hide();
	},
	/** リサイズされたときの処理をします。*/
	onResize: function(){
		dialogAnalyse.content.style.left = (document.body.offsetWidth - dialogAnalyse.content.offsetWidth) / 2
		dialogAnalyse.content.style.top  = (window.innerHeight - dialogAnalyse.content.offsetHeight) / 2
		var bg = document.getElementById("dialogAnalyseBackground");
		if (bg) {
			bg.style.width = document.body.offsetWidth;
			bg.style.height = window.innerHeight;
		}
	},
	/** ダイアログを表示します。*/
	show: function(){
		var bg = document.createElement("div");
		bg.id = "dialogAnalyseBackground";
		bg.style.width = document.body.offsetWidth;
		bg.style.height = window.innerHeight;
		document.body.appendChild(bg);

		this.content.style.position   = "fixed";
		this.content.style.width      = 512;
		this.content.style.visibility = "hidden"
		this.content.style.display    = "block";
		this.selectPage(this.content.childNodes[1].id.replace("Page_", ""));
		this.content.style.left = (document.body.offsetWidth - this.content.offsetWidth) / 2
		this.content.style.top  = (window.innerHeight - this.content.offsetHeight) / 2
		this.content.style.visibility = "visible";
		window.addEventListener("resize", dialogAnalyse.onResize, false);
	},
	/** ダイアログを非表示にします。*/
	hide: function(){
		this.content.style.display = "none";
		var bg = document.getElementById("dialogAnalyseBackground");
		if (bg) document.body.removeChild(bg);
		window.removeEventListener("resize", dialogAnalyse.onResize, false);
	},
	/** ダイアログの内容をクリアします。*/
	clear: function(){
		if (document.getElementById("dialogAnalyse")) {
			document.body.removeChild(document.getElementById("dialogAnalyse"));
			this.content = document.createElement("div");
		}
	},
	/** ダイアログのページを切り替えます。
	 * @param {String} id ページの ID
	 */
	selectPage: function(id){
		var divHeader = document.getElementById("dialogAnalyseHeader");
		for (var i = 0; i < divHeader.childNodes.length - 1; i++) {
			var icon = divHeader.childNodes[i];
			if (icon.id == "Icon_" + id) {
				icon.className = "icon" + " " + "selected";
			} else {
				icon.className = "icon";
			}
		}	
		for (var i = 1; i < this.content.childNodes.length - 1; i++) {
			var page = this.content.childNodes[i];
			if (page.id == "Page_" + id) {
				page.style.display = "block";
			} else {
				page.style.display = "none";
			}
		}
		if ((this.content.offsetTop + this.content.offsetHeight) > window.innerHeight) this.onResize();
	},
	/** ページを追加します。
	 * @param {String} id      ID
	 * @param {String} caption キャプション
	 * @param {String} icon    アイコンの URI
	 */
	addPage: function(id, caption, icon){
		if (!this.content.id) this.content.id = "dialogAnalyse";
		if (!document.getElementById("dialogAnalyse")) document.body.appendChild(this.content);
		var divHeader = document.getElementById("dialogAnalyseHeader");
		if (!divHeader) {
			divHeader = document.createElement("div");
			divHeader.id = "dialogAnalyseHeader";
			divHeader.className = "header";
			var divHeaderEnd = document.createElement("div");
			divHeaderEnd.id = "dialogAnalyseHeaderEnd";
			divHeaderEnd.className = "null";
			divHeader.appendChild(divHeaderEnd);
			this.content.appendChild(divHeader);
			
			var p      = document.createElement("p");
			var ok     = document.createElement("input");
			ok.setAttribute("type",  "submit");
			ok.setAttribute("value", "OK");
			ok.setAttribute("onclick", "dialogAnalyse.onOK()");
			p.appendChild(ok);
			this.content.appendChild(p);
		}
		
		var divIcon = document.createElement("div");
		divIcon.id = "Icon_" + id;
		divIcon.className = "icon";
		divIcon.textContent = caption;
		divIcon.style.backgroundImage = "url(" + icon + ")";
		divIcon.setAttribute("onclick", "dialogAnalyse.selectPage('" + id + "')");
		divHeader.insertBefore(divIcon, document.getElementById("dialogAnalyseHeaderEnd"));
		
		var divPage = document.createElement("div");
		divPage.id = "Page_" + id;
		divPage.style.display = "none";
		this.content.insertBefore(divPage, this.content.lastChild);
	},
	/** フレームを追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 * @param {Number} maxheight 最大の高さ
	 */
	addFrame: function(page, id, caption, maxheight){
		var fieldset = document.createElement("fieldset");
		fieldset.id = "Frame_" + page + "_" + id;
		var legend  = document.createElement("legend");
		legend.textContent = caption;
		fieldset.appendChild(legend);
		var ul = document.createElement("ul");
		if (maxheight) ul.style.maxHeight = maxheight;
		fieldset.appendChild(ul);
		var divPage = document.getElementById("Page_" + page);
		if (divPage) divPage.appendChild(fieldset);
	},
	/** ラベルを追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} frame     フレームの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 * @param {Number} [index]   表示するレス番号へのアンカー
	 * @param {Number} [level]   インデントする階層
	 */
	addLabel: function(page, frame, id, caption, index, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogAnalyseLabel";
		var span = document.createElement("span");
		span.textContent = caption;
		label.appendChild(span);
		if (index) {
			var a = document.createElement("a");
			a.setAttribute("href", "javascript:void(0)");
			a.setAttribute("className", "resPointer");
			a.textContent = "(>>" + index + ")";
			label.appendChild(a);
		}
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** リンクを追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} frame     フレームの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 * @param {String} uri   　　URI
	 * @param {Number} [index]   表示するレス番号へのアンカー
	 * @param {Number} [level]   インデントする階層
	 */
	addLink: function(page, frame, id, caption, uri, index, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogAnalyseLabel";
		var a = document.createElement("a");
		a.setAttribute("href", uri);
		a.setAttribute("target", "_blank");
		a.className = "outLink";
		a.textContent = caption;
		label.appendChild(a);
		if (index) {
			label.appendChild(document.createTextNode(" ("));
			var a = document.createElement("a");
			a.setAttribute("href", "javascript:void(0)");
			a.className = "resPointer";
			a.textContent = ">>" + index;
			label.appendChild(a);
			label.appendChild(document.createTextNode(")"));
		}
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** ID を追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} frame     フレームの ID
	 * @param {String} id        ID
	 * @param {String} caption   キャプション
	 * @param {Number} count   　ID の発言回数
	 * @param {Number} [level]   インデントする階層
	 */
	addID: function(page, frame, id, caption, count, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogAnalyseLabel";
		/*
		var a = document.createElement("a");
		a.setAttribute("href", "javascript:void(0)");
		a.setAttribute("target", "_blank");
		a.className = "mesID_" + caption;
		a.textContent = "ID:" + caption;
		label.appendChild(a);
		*/
		var span = document.createElement("span");
		span.className = "mesID_" + caption + " resID2";
		span.textContent = "ID:" + caption;
		label.appendChild(span)
		label.appendChild(document.createTextNode("(" + count + ")"));
		li.appendChild(label);
		parentObject.appendChild(li);
	},
	/** 逆参照を追加します。
	 * @param {String} page      追加するページの ID
	 * @param {String} frame     フレームの ID
	 * @param {String} id        ID
	 * @param {Number} index     レス番号
	 * @param {Number} count   　レス番号に対する言及回数
	 * @param {Number} [level]   インデントする階層
	 */
	addTrackback: function(page, frame, id, index, count, level){
		var parentObject = document.getElementById("Frame_" + page + "_" + frame).lastChild;
		if (level > 0) {
			for (var i = 1; i <= level; i++) {
				parentObject.appendChild(document.createElement("ul"));
				parentObject = parentObject.lastChild;
			}
		}
		var li = document.createElement("li");
		var label = document.createElement("label");
		label.className = "dialogAnalyseLabel";
		var a = document.createElement("a");
		a.setAttribute("href", "javascript:void(0)");
		//a.setAttribute("target", "_blank");
		a.className = "resPointer";
		a.textContent = ">>" + index;
		label.appendChild(a);
		label.appendChild(document.createTextNode("(" + count + ")"));
		li.appendChild(label);
		parentObject.appendChild(li);
	}
};